var undefined; //for fast tests for undefined values
function SQLite(db,mode) {
  if (!(this instanceof SQLite)) return new SQLite(db);
    var file = Components.classes["@mozilla.org/file/directory_service;1"]
    .getService(Components.interfaces.nsIProperties)
    .get("ProfD", Components.interfaces.nsIFile);
    file.append(db);
  this.conn = Components.classes["@mozilla.org/storage/service;1"]
    .getService(Components.interfaces.mozIStorageService).openDatabase(file);
  return this;
}
SQLite.prototype.queries={};
SQLite.prototype.queryFunction=function (name,query) {
  //If name is an object
  if (typeof name == 'object') {
    //initialize our return value
    var queryGroup={};
    //for each function NVP
    for (var i in name)
      //create a matching queryFunction
      queryGroup[i]=arguments.callee.apply(this,[i,name[i]]);
    //return the query namespace
    return queryGroup;
  }
  var matchAll = function (str,re,set) {
    //Required for SQLite parameter matching
    if (set==undefined) set=0;
    //copy source string, initialize output array
    var ret=[],ts = str;
    while (re.test(ts)) {
      //match
      ret.push(ts.match(re));
      //remove match
      ts=ts.replace(re,'');
    }
    //If we have to swap orders, do so.
    if (set==1) {
      var i,j,nRet = [];
      for (i=0; i<ret.length; i++) {
        for (j=0; j<ret[i].length; j++) {
          if (i==0) nRet[j]=[];
          nRet[j].push(ret[i][j]);
        }
      }
      ret=nRet;
    }
    return ret;
  };
  var
    //allow the internal function to access this SQLite instance
    _this=this,
    //use _conn as a shortcut if necessary
    _conn=this.conn,
    //Match all % parameters (for type matching)
    params=matchAll(query, /%(n|s|f)(\d+)/i),
    //And replace with their real counterparts
    realQuery=query.replace(/%(n|s|f)(\d+)/ig,'?$2'),
    //create the statement
    statement=_conn.createStatement(realQuery),
    //Create the query function, loaded with the variables available above.
    exec = function () {
      var i,idx;
      //walk through the parameters
      for (i=0; i<params.length; i++) {
        //get the param index for the parameter
        idx=statement.getParameterIndex('?'+params[i][2]);
        switch (params[i][1].toLowerCase()) {
          case 'n':
            //it's a number.  Use int64's
            statement.bindInt64Parameter(idx,arguments[idx]);
            break;
          case 'f':
            //it's a float.  Use doubles.
            statement.bindDoubleParameter(idx,arguments[idx]);
            break;
          default:
            //Assume it's a UTF8 string; ASCII and blobs will handle this fine on the input side
            statement.bindUTF8StringParameter(idx,arguments[idx]);
        }
      }
      //Some variables for the return
      var
        cols = statement.columnCount,
        col = 0,
        colTypes=[],
        colNames=[],
        ret=[],
        name,
        rowData;
      //If we've requested columns
      if (cols>0) {
        while (statement.executeStep()) {
          //execute like we're expecting a return value.
          rowData={};
          for (col=0; col<cols; col++) {
            if (colNames[col]==undefined) {
              //populate the colNames and colTypes arrays, as needed
              colNames[col]=statement.getColumnName(col);
              colTypes[col]=statement.getTypeOfIndex(col);
            }
            name=colNames[col];
            switch (colTypes[col]) {
              //Depending on the column type (this really matters on output)
              //fill the rowData object
              case 0:
                rowData[name]=null; break;
              case 1:
                rowData[name]=statement.getInt64(col); break;
              case 2:
                rowData[name]=statement.getDouble(col); break;
              case 3:
                rowData[name]=statement.getUTF8String(col); break;
              case 4:
                rowData[name]=statement.getBlob(col); break;
            }
          }
          //And, once this row is done, append it to the array.
          ret.push(rowData);
        }
      } else {
        //If there's no columns to grab, just run the query
        statement.execute();
      }
      //Either way, reset to stave off hobgoblins
      statement.reset();
      if (ret.length>0) //We've got return values?  Good!  Send 'em up.
        return ret;
      //We don't?  Well, an insert will return the last insert ID
      //and a no-value select will return 0.
      return _conn.lastInsertRowID;
    };
  //Add exec to our set of precached queries, and return it.
  return this.queries[name]=exec;
};